// Pins the LEDs and Buttons are connected to
const unsigned char RED_LED[3] =   {3,  6, A4};
const unsigned char GREEN_LED[3] = {2, 10, A2};
const unsigned char RED_BTN[3] =   {5,  8, A3};
const unsigned char GREEN_BTN[3] = {4, 12, A1};

#define MODE_DISABLED 0
#define MODE_FLASHING 1
#define MODE_STATIC   2

// The current state of each button
struct ButtonState {
  unsigned char ledPin;           // Which pin is the LED pin
  unsigned char btnPin;           // Which pin is the button pin
  char shortPress, longPress;     // Characters to send for short and long presses
  unsigned char mode;             // The current mode this button is in. 
  bool triggered, longTriggered;  // If the button is triggered or not
  unsigned long triggerTime;      // When it triggered
};

// The 6 buttons
ButtonState buttons[6];

// Prepare
void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(115200);
  while (!Serial) {};
  
  // Prepare buttons
  for (unsigned char c=0; c<3; c++) {
    buttons[c].ledPin = RED_LED[c];
    buttons[c+3].ledPin = GREEN_LED[c];
    buttons[c].btnPin = RED_BTN[c];
    buttons[c+3].btnPin = GREEN_BTN[c];
    pinMode(RED_LED[c], OUTPUT);
    pinMode(GREEN_LED[c], OUTPUT);
    pinMode(RED_BTN[c], INPUT_PULLUP);
    pinMode(GREEN_BTN[c], INPUT_PULLUP);
    buttons[c].shortPress = 'A'+c;
    buttons[c+3].shortPress = 'D'+c;
    buttons[c].longPress = 'G'+c;
    buttons[c+3].longPress = 'J'+c;
    buttons[c].mode = MODE_DISABLED;
    buttons[c+3].mode = MODE_DISABLED;
    buttons[c].triggered = false;
    buttons[c+3].triggered = false;
    buttons[c].longTriggered = false;
    buttons[c+3].longTriggered = false;
  }
}

// Manual PWM code
int readStatus = 99;
bool rampDirection = true;
unsigned char rampTarget = 255;
unsigned char rampCounter = 32;

void loop() {
  // PWM calculator
  unsigned char lastCounter = rampCounter;
  rampCounter+=32;
  if (lastCounter > rampCounter) {
    if (rampTarget == 255) rampDirection = false;
    if (rampTarget == 32) rampDirection = true;
    rampTarget+=(rampDirection?1:-1);
  }

  // PWM Output
  bool flashMode = rampCounter > rampTarget;

  // Did the PC send us any data?
  while (Serial.available()) {
    char tmp[2];
    tmp[0] = Serial.read();
    if (tmp[0] == '!') {   // Test if we exist
      Serial.write('Z');
    } else
    if (tmp[0] == '@') readStatus = 0; else {  // Reset Position
      if (readStatus<6) {
        tmp[1] = '\0';
        buttons[readStatus++].mode = atoi(tmp);   // Read button status
      } 
    }
  }

  // Update buttons
  for (unsigned char btn=0; btn<6; btn++) {
    // Control the button LED
    digitalWrite(buttons[btn].ledPin, (buttons[btn].mode==MODE_FLASHING)?flashMode:((buttons[btn].mode==MODE_STATIC)?HIGH:LOW));
    if (buttons[btn].mode) {
      if (digitalRead(buttons[btn].btnPin) == LOW) {
        if (!buttons[btn].triggered) {
          buttons[btn].longTriggered = false;
          buttons[btn].triggered = true;
          buttons[btn].triggerTime = millis();
          if (buttons[btn].mode == MODE_FLASHING) Serial.write(buttons[btn].shortPress);
        } else {          
          if ((millis() - buttons[btn].triggerTime>1500) && (!buttons[btn].longTriggered)) {
            if (buttons[btn].mode == MODE_STATIC) Serial.write(buttons[btn].longPress);
            buttons[btn].longTriggered = true;
          }
        }
      } else {
        if (buttons[btn].triggered) {
          // undo after a short pause
          if (millis() - buttons[btn].triggerTime>200) buttons[btn].triggered = false;
        }
      }
    }
  }  
}
